/*
 * Copyright 2007 ZXing authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.zxing.qrcode.detector;

import com.google.zxing.ResultPoint;
import enerj.lang.*;

/**
 * <p>Encapsulates an alignment pattern, which are the smaller square patterns found in
 * all but the simplest QR Codes.</p>
 *
 * @author Sean Owen
 */
public final class AlignmentPattern extends ResultPoint {

  private final @Approx(0.95) float estimatedModuleSize;

  AlignmentPattern(float posX, float posY, @Approx float estimatedModuleSize) {
    super(posX, posY);
    this.estimatedModuleSize = estimatedModuleSize;
  }

  /**
   * <p>Determines if this alignment pattern "about equals" an alignment pattern at the stated
   * position and size -- meaning, it is at nearly the same center with nearly the same size.</p>
   */
  @Approx boolean aboutEquals(@Approx(0.99) float moduleSize, float i, float j) {
    float abs1 = Math.abs(i - getY());
    float abs2 = Math.abs(j - getX());
    @Approx(0.95) boolean cond = abs1 <= moduleSize && abs2 <= moduleSize;
    if (Endorsements.endorse(cond)) {
      @Approx(0.94) float approxDiff = moduleSize - estimatedModuleSize;
      float moduleSizeDiff = Math.abs(Endorsements.endorse(approxDiff));
      @Approx boolean ret = moduleSizeDiff <= 1.0f || moduleSizeDiff / estimatedModuleSize <= 1.0f;
      return ret;
    }
    return false;
  }

}
